//brk.c
//Error interrupt handling in real-mode
//Bryan E. Topp <betopp@betopp.com> 2023

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include "brk.h"
#include <i86.h>
#include <dos.h>
#include <stdint.h>

//Break handler - do nothing, consume ctrl-C
static void __interrupt __far brk_isr_break(void)
{
	return;
}

//Critical error handler - tell DOS to return failure code to the caller (us)
static uint8_t __interrupt __far brk_isr_criticalerr(void)
{
	return 3; //Fail
}

void brk_init(void)
{
	//Don't need to chain or save old vectors - DOS handles restoring for us
	_dos_setvect(0x23, brk_isr_break);
	_dos_setvect(0x24, (void (__interrupt *)())brk_isr_criticalerr);
}
