//mouse.c
//Mouse callback for Stump real-mode loop
//Bryan E. Topp <betopp@betopp.com> 2022

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include "mouse.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <i86.h>

//Called by mouse driver when mouse state changes.
#pragma off(check_stack)
void _loadds far mouse_callback(short btn, short xrel, short yrel)
{
	//Called by mouse driver with information in registers
	#pragma aux mouse_callback parm [BX] [SI] [DI]
	
	//Send mouse event to kernel
	//printf("mouse %X %d %d\n", btn, xrel, yrel); //Test - print to screen (doesn't work under FreeDOS)
	//*(uint16_t far*)(MK_FP(0xb800, (yrel * 160) + (xrel * 2))) = 0xFFFF; //Test - draw on screen
	(void)btn;
	(void)xrel;
	(void)yrel;
}
#pragma on(check_stack)

//Deregisters mouse callback at exit.
static void mouse_atexit(void)
{
	//Deregister mouse callback
	union REGS regs = {0};
	struct SREGS sregs = {0};
	segread(&sregs);
	regs.w.ax = 0xC;
	regs.w.cx = 0;
	sregs.es = 0;
	regs.w.dx = 0;
	int86x(0x33, &regs, &regs, &sregs);
}

void mouse_init(void)
{
	//Check for presence of MS Mouse API
	union REGS regs = {0};
	struct SREGS sregs = {0};
	regs.w.ax = 0;
	int86(0x33, &regs, &regs);
	if(regs.w.ax != 0xFFFF)
	{
		printf("No mouse driver installed.\n");
		exit(-1);
	}
	
	//Register our callback with the mouse driver
	segread(&sregs);
	regs.w.ax = 0xC;
	regs.w.cx = 0xFFFF; //All events
	sregs.es = _FP_SEG(&mouse_callback);
	regs.w.dx = _FP_OFF(&mouse_callback);
	int86x(0x33, &regs, &regs, &sregs);
	
	//Clean up mouse driver on exit
	atexit(&mouse_atexit);
}
