//ring.h
//Ringbuffers for DOS loader communication with kernel
//Bryan E. Topp <betopp@betopp.com> 2022

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#ifndef RING_H
#define RING_H

#include <stdint.h>

//Channel numbers that we use - make sure this lines up with the kernel
typedef enum dring_channel_e
{
	DRING_CHANNEL_NONE = 0,
	
	DRING_CHANNEL_PANIC,
	
	DRING_CHANNEL_MEMRANGES,
	
	DRING_CHANNEL_ENET0_SEND,
	DRING_CHANNEL_ENET0_RECV,
	
	DRING_CHANNEL_UART0_SEND,
	DRING_CHANNEL_UART0_RECV,
	
	DRING_CHANNEL_VMEMS,
	DRING_CHANNEL_VMODE,
	DRING_CHANNEL_VCURR,
	
	DRING_CHANNEL_INPUT,
	
	DRING_CHANNEL_MAX
	
} dring_channel_t;

//Initializes ringbuffers and maps them in kernel-space.
void ring_init(uint64_t *pml4);

//Writes to a ring. Returns the number of bytes written.
int ring_write(int channel, const void *data_ptr, int data_len);

//Reads from a ring. Returns the number of bytes read.
int ring_read(int channel, void *buf_ptr, int buf_len);

#endif //RING_H
