//xmem.h
//Physical extended-memory allocation for Stump real-mode loader
//Bryan E. Topp <betopp@betopp.com> 2022

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#ifndef XMEM_H
#define XMEM_H

#include <stdint.h>

//Requests and locks memory from XMS, putting it in our pools
void xmem_init(void);

//Allocates a page of memory from our pools, returning its physical address.
uint32_t xmem_palloc(void);

//Fills a page from a memory pool, by physical address, with data from real-mode space.
void xmem_pfill(uint32_t page, const void * far data);

//Reads a 64-bit value from a page allocated from our pools, by physical address.
uint64_t xmem_read64(uint32_t addr);

//Writes a 64-bit value to a page allocated from our pools, by physical address.
void xmem_write64(uint32_t addr, uint64_t value);

//Writes all remaining allocations to a kernel ringbuffer, so the kernel can use them.
void xmem_handoff(void);


#endif //XMEM_H
