//amd64.h
//Common definitions for AMD64 machine support
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#ifndef AMD64_H
#define AMD64_H

#include "shared/kstd.h"

//Sets the Page Directory Base Register
static inline void setcr3(uint64_t val) { asm volatile ("mov %%rax, %%cr3": :"a" (val)); }

//Returns the Page Directory Base Register
static inline uint64_t getcr3(void) { uint64_t val; asm volatile ("mov %%cr3, %%rax": "=a"(val) : ); return val; }

//Invalidates the given page by virtual address
static inline void invlpg(uint64_t vaddr) { asm volatile ("invlpg (%%rax)": : "a" (vaddr)); }

//In/out instruction wrappers
static inline void outb(uint16_t port, uint8_t byte)
{
	asm volatile ("outb %%al,%%dx": :"d" (port), "a" (byte));
}

static inline void outw(uint16_t port, uint16_t word)
{
	asm volatile ("outw %%ax,%%dx": :"d" (port), "a" (word));
}

static inline void outd(uint16_t port, uint32_t dword)
{
	asm volatile ("outl %%eax,%%dx": :"d" (port), "a" (dword));
}

static inline uint8_t inb(uint16_t port)
{
	uint8_t byte;
	asm volatile ("inb %%dx,%%al":"=a"(byte):"d"(port));
	return byte;
}

static inline uint16_t inw(uint16_t port)
{
	uint16_t word;
	asm volatile ("inw %%dx,%%ax":"=a"(word):"d"(port));
	return word;
}

static inline uint32_t ind(uint16_t port)
{
	uint32_t dword;
	asm volatile ("inl %%dx,%%eax":"=a"(dword):"d"(port));
	return dword;
}

#endif //AMD64_H
