/* kernel.ld */
/* GNU linker script for building 64-bit kernel */
/* Bryan E. Topp <betopp@betopp.com> 2021 */

ENTRY(stubboot)

SECTIONS
{
	/* Difference between physical and virtual addresses as-linked. */
	/* Put the kernel in the top 1GByte so we can use negative 32-bit offsets. */
	/* Use only 1GByte so we only need one PAE Page Directory. */
	_KERNEL_VOFFS = 0xFFFFFFFFC0000000;
	. = _KERNEL_VOFFS;
	
	.boot ALIGN(0x1000) : AT(ADDR(.boot) - _KERNEL_VOFFS)
	{	
		/* Put trampoline up-front so it gets copied by the loader */
		*(.stubboot*)
	}
	
	.text ALIGN(0x1000) : AT(ADDR(.text) - _KERNEL_VOFFS)
	{	
		*(.eh_frame*)
		*(.text*)
	}
	
	.rodata ALIGN(0x1000) : AT(ADDR(.rodata) - _KERNEL_VOFFS)
	{	
		*(.rodata)
		
		/* System TAR needs to be page-aligned so we can free it after unpacking */
		. = ALIGN(0x1000);
		_SYSTAR_START = .;
		*(.systar*)
		. = ALIGN(0x1000);
		_SYSTAR_END = .;
	}
	
	.data ALIGN(0x1000) : AT(ADDR(.data) - _KERNEL_VOFFS)
	{	
		*(.data)
		
		. += 1;
		. = ALIGN(0x1000);
	}
	
	.bss ALIGN(0x1000) : AT(ADDR(.bss) - _KERNEL_VOFFS)
	{
		*(.bss)
		*(COMMON)
	}
	
	/*Pad end-of-kernel to page boundary*/
	.endpad ALIGN(0x1000) : AT(ADDR(.endpad) - _KERNEL_VOFFS)
	{
	
	}
	
	_BSS_END_ADDR = .;
}
