;m_intr.asm
;Interrupt control on AMD64
;Bryan E. Topp <betopp@betopp.com> 2021

;This file is part of Stump; Copyright Bryan E. Topp, 2023.
;Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
;Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
;You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

global m_intr_ei ;void m_intr_ei(bool enable);
m_intr_ei:
	;Do you want the moustache on or off?
	cmp RDI, 0
	jne .enable
		cli
		ret
	.enable:
		sti
		ret

global m_intr_halt ;void m_intr_halt(void);
m_intr_halt:
	;On AMD64, "sti" enables interrupts after execution of the following instruction.
	;So a sequence of sti/hlt will atomically "halt with interrupts enabled".
	sti
	hlt
	ret

global m_intr_wake ;void m_intr_wake(void);
m_intr_wake:

	;Get APIC base address in RAX (todo - can we assume this is constant? man I want x2apic with its own MSRs)
	mov ECX, 0x1B ;APIC BAR
	rdmsr
	and EAX, 0xFFFFF000
	and EDX, 0x000FFFFF
	shl RDX, 32
	or RAX, RDX
	
	;Trigger interprocessor interrupt
	lea RDI, [RAX + 0x300] ;interrupt command register low
	mov RSI, 0xC40FE ;Fixed interrupt, positive edge-trigger, to all-except-self, vector 0xFE
	extern pspace_write32
	call pspace_write32
	
	ret
	
global m_intr_ticks ;void m_intr_ticks(void);
m_intr_ticks:
	rdtsc
	and EAX, 0xFFFFFFFF
	shl RDX, 32
	or RAX, RDX
	shr RAX, 22 ;Assume a ~4GHz TSC
	ret
