;m_spl.asm
;Spinlock functions on AMD64
;Bryan E. Topp <betopp@betopp.com> 2021

;This file is part of Stump; Copyright Bryan E. Topp, 2023.
;Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
;Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
;You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

section .text
bits 64

global m_spl_acq ;void m_spl_acq(m_spl_t *spl);
m_spl_acq:
	mov AX, 0x0100 ;AL = 0, AH = 1.
	lock cmpxchg [RDI], AH ;Compares AL (0) with the spinlock. If it's 0, AH (1) is stored.
	jnz .wait ;If we failed, ZF is cleared - wait and try again
		mfence ;Make sure all prior memory operations complete before we say we "have the lock".
		ret
	.wait:
		;Didn't get the lock.
		;Use non-locked accesses to wait until the lock seems to be 0. Then try again.
		pause
		cmp byte [RDI], 0
		jne .wait
		jmp m_spl_acq

global m_spl_try ;bool m_spl_try(m_spl_t *spl);
m_spl_try:
	mov AX, 0x0100 ;AL = 0, AH = 1.
	lock cmpxchg [RDI], AH ;Compares AL (0) with the spinlock. If it's 0, AH (1) is stored.
	jnz .failed ;If we failed, ZF is cleared - return false
		mfence ;Make sure all prior memory operations complete before we say we "have the lock".
		mov RAX, 1 ;Return true
		ret
	.failed:
		mov RAX, 0 ;Return false
		ret

global m_spl_rel ;void m_spl_rel(m_spl_t *spl);
m_spl_rel:
	;Just set the lock value to 0
	mov byte [RDI], 0
	ret

global m_cache_finish
m_cache_finish:
	mfence
	ret