//m_drop.h
//Dropping to usermode from kernel
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#ifndef M_DROP_H
#define M_DROP_H

//User-mode CPU context - always like ARMv4t.
typedef struct m_drop_s
{
	//Should really be defined per machine... needs to just be large enough for machine-specific code.
	uint64_t words[24];
} m_drop_t;

//Ensures that a drop structure is safe for use as a user context
void m_drop_sanitize(m_drop_t *drop);

//Sets the program counter in the given user context.
void m_drop_entry(m_drop_t *drop, uint64_t entry);

//Sets a parameter in the given user context.
void m_drop_param(m_drop_t *drop, int which, uint64_t value);

//Sets the return-value in the given user context.
void m_drop_retval(m_drop_t *drop, uint64_t retval);

//Drops to userspace using the given context.
void m_drop(const void *drop) __attribute__((noreturn));


//Returns a description of the user-mode architecture.
const char *m_drop_archname(void);

//Returns the correct ELF machine-ID word for the user-mode architecture.
uint16_t m_drop_elfmachine(void);

//Returns the correct ELF class byte for the user-mode architecture.
uint8_t m_drop_elfclass(void);


#endif //M_DROP_H

