//m_spl.h
//Spinlock functions, per-machine
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#ifndef M_SPL_H
#define M_SPL_H

#include "../src/kstd.h"

//Type representing a spinlock
typedef volatile int m_spl_t;

//Acquires a spinlock, blocking until it is held.
void m_spl_acq(m_spl_t *spl);

//Attempts to acquire a spinlock, but doesn't block if it is already in use.
//Returns true if the spinlock was acquired, false otherwise.
bool m_spl_try(m_spl_t *spl);

//Releases the given spinlock.
void m_spl_rel(m_spl_t *spl);

#endif //M_SPL_H
