//devs.h
//Built-in character device functions
//Bryan E. Topp <betopp@betopp.com> 2022

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#ifndef DEVS_H
#define DEVS_H

#include "kstd.h"

//Notifies the given device that it is being opened.
//Returns 0 on success or a negative error number.
int devs_open(int rdev);

//Notifies the given device that it is being closed.
void devs_close(int rdev);

//Reads from the given device.
ssize_t devs_read(int rdev, void *buf, ssize_t nbytes);

//Writes to the given device.
ssize_t devs_write(int rdev, const void *buf, ssize_t nbytes);

//Unblocks threads waiting on the given UART.
void devs_isr_uart(int which);

//Unblocks threads waiting on the given Ethernet interface.
void devs_isr_enet(int which);

#endif //DEVS_H
