//kstd.h
//Pieces of standard library that we use in kernel - internally defined
//Bryan E. Topp <betopp@betopp.com> 2022

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#ifndef KSTD_H
#define KSTD_H

#ifdef HOSTED
	#include <sys/types.h>
	#include <stddef.h>
	#include <stdbool.h>
	#include <stdint.h>
	#include <string.h>
#else
	//stdint replacement
	typedef __INTPTR_TYPE__ intptr_t;
	typedef __INT8_TYPE__ int8_t;
	typedef __INT16_TYPE__ int16_t;
	typedef __INT32_TYPE__ int32_t;
	typedef __INT64_TYPE__ int64_t;
	typedef __UINTPTR_TYPE__ uintptr_t;
	typedef __UINT8_TYPE__ uint8_t;
	typedef __UINT16_TYPE__ uint16_t;
	typedef __UINT32_TYPE__ uint32_t;
	typedef __UINT64_TYPE__ uint64_t;

	typedef __PTRDIFF_TYPE__ ptrdiff_t;

	typedef __SIZE_TYPE__ size_t;
	typedef long ssize_t;

	//stdbool replacement
	typedef _Bool bool;
	#define false 0
	#define true 1

	//sys types
	typedef int pid_t;
	typedef int off_t;
	typedef int dev_t;
	typedef int mode_t;
	typedef int ino_t;

	//Standard-library functions that we do
	char *strncpy(char *dst, const char *src, size_t n);
	size_t strlen(const char *s);
	size_t strnlen(const char *s, size_t maxlen);
	int strcmp(const char *s1, const char *s2);
	char *strchr(const char *s, int c);
	int memcmp(const void *s1v, const void *s2v, size_t len);
	void *memset(void *dst, int c, size_t len);
	void *memcpy(void *dst, const void *src, size_t len);
	void *memmove(void *dst, const void *src, size_t len);

	//Constants
	#define NULL ((void*)(0))	
#endif

//Define error numbers as system call library expects
#include "sc_errno.h"
#define EPERM _SC_EPERM
#define ENOENT _SC_ENOENT
#define ESRCH _SC_ESRCH
#define EINTR _SC_EINTR
#define EIO _SC_EIO
#define ENXIO _SC_ENXIO
#define E2BIG _SC_E2BIG
#define ENOEXEC _SC_ENOEXEC
#define EBADF _SC_EBADF
#define ECHILD _SC_ECHILD
#define EAGAIN _SC_EAGAIN
#define EWOULDBLOCK _SC_EWOULDBLOCK
#define ENOMEM _SC_ENOMEM
#define EACCES _SC_EACCES
#define EFAULT _SC_EFAULT
#define EBUSY _SC_EBUSY
#define EEXIST _SC_EEXIST
#define EXDEV _SC_EXDEV
#define ENODEV _SC_ENODEV
#define ENOTDIR _SC_ENOTDIR
#define EISDIR _SC_EISDIR
#define EINVAL _SC_EINVAL
#define ENFILE _SC_ENFILE
#define EMFILE _SC_EMFILE
#define ENOTTY _SC_ENOTTY
#define ETXTBSY _SC_ETXTBSY
#define EFBIG _SC_EFBIG
#define ENOSPC _SC_ENOSPC
#define ESPIPE _SC_ESPIPE
#define EROFS _SC_EROFS
#define EMLINK _SC_EMLINK
#define EPIPE _SC_EPIPE
#define EDOM _SC_EDOM
#define ERANGE _SC_ERANGE
#define EDEADLK _SC_EDEADLK
#define EDEADLOCK _SC_EDEADLOCK
#define ENAMETOOLONG _SC_ENAMETOOLONG
#define ENOLCK _SC_ENOLCK
#define ENOSYS _SC_ENOSYS
#define ENOTEMPTY _SC_ENOTEMPTY
#define ELOOP _SC_ELOOP
#define ENOMSG _SC_ENOMSG
#define EIDRM _SC_EIDRM

#endif //KSTD_H
