//kstring.c
//String manipulation functions for kernel
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include "kstd.h"

char *strncpy(char *dst, const char *src, size_t n)
{
	size_t ii = 0;
	while(ii < n)
	{
		if(src[ii] == '\0')
			break;
		
		dst[ii] = src[ii];
		ii++;
	}
	
	while(ii < n)
	{
		dst[ii] = 0;
		ii++;
	}
	
	return dst;
}

size_t strlen(const char *s)
{
	size_t l = 0;
	while(s[l] != '\0')
	{
		l++;
	}
	return l;
}

size_t strnlen(const char *s, size_t maxlen)
{
	size_t l = 0;
	while(s[l] != '\0' && l < maxlen)
	{
		l++;
	}
	return l;
}

int strcmp(const char *s1, const char *s2)
{
	while(1)
	{
		if(*s1 != *s2)
			return *s1 - *s2;
		
		if(*s1 == '\0')
			return 0;
		
		s1++;
		s2++;
	}
}

char *strchr(const char *s, int c)
{
	while(1)
	{
		if(*s == c)
			return (char*)s; //Note - checked before end-of-string, so c can be \0
		
		if(*s == '\0')
			return NULL;
		
		s++;
	}
}

int memcmp(const void *s1v, const void *s2v, size_t len)
{
	const char *s1 = (const char*)s1v;
	const char *s2 = (const char*)s2v;
	
	for(size_t ll = 0; ll < len; ll++)
	{
		if(s1[ll] != s2[ll])
			return s1[ll] - s2[ll];
	}
	
	return 0;
}

void *memset(void *dst, int c, size_t len)
{
	uint8_t *dstb = (uint8_t*)dst;
	
	while( ((uintptr_t)dstb & 0x3) && (len > 0) )
	{
		*dstb = c & 0xFF;
		dstb++;
		len--;
	}
	
	uint32_t *dstw = (uint32_t*)dstb;
	while(len >= 4)
	{
		*dstw = (c & 0xFF) | ((c & 0xFF) << 8) | ((c & 0xFF) << 16) | ((c & 0xFF) << 24);
		dstw++;
		len -= 4;
	}
	
	dstb = (uint8_t*)dstw;
	while(len)
	{
		*dstb = c & 0xFF;
		dstb++;
		len--;
	}
	return dst;
}

void *memcpy(void *dst, const void *src, size_t len)
{	
	if( ((uintptr_t)dst | (uintptr_t)src | (uintptr_t)len) & 0x3 )
		return memmove(dst, src, len);
	
	const uint32_t *srcw = src;
	uint32_t *dstw = dst;
	len /= 4;
	while(len > 0)
	{
		*(dstw++) = *(srcw++);
		len--;
	}
	
	return dst;
}

void *memmove(void *dst, const void *src, size_t len)
{
	const char *sb = (const char*)src;
	char *db = (char*)dst;
	if(db > sb)
	{
		for(size_t ii = len; ii > 0; ii--)
		{
			db[ii-1] = sb[ii-1];
		}
	}
	else
	{
		for(size_t ii = 0; ii < len; ii++)
		{
			db[ii] = sb[ii];
		}
	}
	return dst;
}
