//ramfs.h
//In-memory filesystem
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#ifndef RAMFS_H
#define RAMFS_H

#include "kstd.h"
#include "sc.h"

//Initializes in-memory filesystem
void ramfs_init(void);


//Locks the filesystem. Do this before operating on it on your thread.
void ramfs_lock(void);

//Unlocks the filesystem. Do this after operating on it on your thread.
void ramfs_unlock(void);


//Makes a new inode.
//Returns 0 on success or a negative error number.
int ramfs_make(int dir, const char *name, mode_t mode, dev_t special, int *ino_out);

//Finds an existing inode.
//Returns 0 on success or a negative error number.
int ramfs_find(int dir, const char *name, int *ino_out);

//Removes a directory entry, optionally from a specific inode.
int ramfs_unlink(int dir, const char *name, int rmino);

//Reads from an inode. Returns the number of bytes read or a negative error number.
ssize_t ramfs_read(int ino, off_t off, void *buf, ssize_t len, bool back);

//Writes into an inode. Returns the number of bytes written or a negative error number.
ssize_t ramfs_write(int ino, off_t off, const void *buf, ssize_t len, bool back);

//Truncates the given file to the given length.
off_t ramfs_trunc(int ino, off_t size);

//Returns status information about the given file.
int ramfs_stat(int ino, _sc_stat_t *st);


//Increments the reference-count of open files (and possibly, readers and/or writers) on the given inode.
void ramfs_inc(int ino, int access);

//Decrements the reference-count of open files (and possibly, readers and/or writers) on the given inode. May free it.
void ramfs_dec(int ino, int access);


#endif //RAMFS_H
