//syscalls_sig.c
//System call declarations on kernel-side - system info
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include "syscalls.h"
#include "thread.h"
#include "process.h"
#include "kassert.h"
#include "kstd.h"

int k_sc_info_enum(int inpid, int prev)
{
	if(inpid < 0 || prev < 0)
		return -EINVAL;
	
	if(inpid == 0)
	{
		//Looking for processes in the system
		for(int idx = (prev % PROCESS_MAX); idx < PROCESS_MAX; idx++)
		{
			process_t *pptr = &(process_table[idx]);
			m_spl_acq(&(pptr->spl));
			if(pptr->state != PROCESS_STATE_NONE && pptr->pid != prev)
			{
				KASSERT( (pptr->pid % PROCESS_MAX) == idx );
				pid_t retval = pptr->pid;
				m_spl_rel(&(pptr->spl));
				return retval;
			}
			m_spl_rel(&(pptr->spl));
		}
		return 0;
	}
	else
	{
		//Check if the given process exists at all
		bool pid_valid = false;
		for(int idx = 0; idx < PROCESS_MAX; idx++)
		{
			process_t *pptr = &(process_table[idx]);
			m_spl_acq(&(pptr->spl));
			if(pptr->state != PROCESS_STATE_NONE && pptr->pid == inpid)
			{
				pid_valid = true;
			}
			m_spl_rel(&(pptr->spl));
			
			if(pid_valid)
				break;
		}
		
		if(!pid_valid)
			return -ESRCH;
		
		//Looking for threads in the given process
		for(int idx = (prev % THREAD_MAX); idx < THREAD_MAX; idx++)
		{
			thread_t *tptr = &(thread_table[idx]);
			m_spl_acq(&(tptr->spl));
			if(tptr->state != THREAD_STATE_NONE && tptr->tid != prev && tptr->process->pid == inpid)
			{
				KASSERT( (tptr->tid % PROCESS_MAX) == idx );
				pid_t retval = tptr->tid;
				m_spl_rel(&(tptr->spl));
				return retval;
			}
			m_spl_rel(&(tptr->spl));
		}
		return 0;
	}
}	
	
int k_sc_info_cpu(_sc_info_cpu_t *buf_ptr, int buf_len)
{
	//Build the info in kernel-space
	_sc_info_cpu_t kbuf = {0};
	
	//Todo...
	
	//Copy the appropriate amount out to the user
	if(buf_len > (int)sizeof(kbuf))
		buf_len = sizeof(kbuf);
	
	return process_memput(buf_ptr, &kbuf, buf_len);
}

int k_sc_info_mem(_sc_info_mem_t *buf_ptr, int buf_len)
{
	//Build the info in kernel-space
	_sc_info_mem_t kbuf = {0};
	
	//Todo...
	
	//Copy the appropriate amount out to the user
	if(buf_len > (int)sizeof(kbuf))
		buf_len = sizeof(kbuf);
	
	return process_memput(buf_ptr, &kbuf, buf_len);
}
