//waiters.c
//Lists of threads waiting to be woken
//Bryan E. Topp <betopp@betopp.com> 2022

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include "waiters.h"
#include "thread.h"

void waiters_addcur(pid_t *waiters_array, size_t waiters_bytes)
{
	pid_t curtid = thread_curtid();
	
	//Note that this thread has something meaningful to wait on
	thread_table[curtid % THREAD_MAX].pause_wake_queued = true;
	
	for(size_t ww = 0; ww < (waiters_bytes / sizeof(waiters_array[0])); ww++)
	{
		if(waiters_array[ww] == 0 || waiters_array[ww] == curtid)
		{
			waiters_array[ww] = curtid;
			return;
		}
	}
	
	//No room left in this array of waiters. Unpause one of them and take its place.
	//Ugly but technically correct. They'll waste some CPU cycles and re-pause. It'll thrash.
	thread_unpause(waiters_array[0]);
	waiters_array[0] = curtid;
	return;
}

void waiters_release(pid_t *waiters_array, size_t waiters_bytes)
{
	for(size_t ww = 0; ww < (waiters_bytes / sizeof(waiters_array[0])); ww++)
	{
		if(waiters_array[ww])
			thread_unpause(waiters_array[ww]);
		
		waiters_array[ww] = 0;
	}
}

