//sc_errno.h
//System call library - error numbers
//Bryan E. Topp <betopp@betopp.com> 2022

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#ifndef _SC_ERRNO_H
#define _SC_ERRNO_H

//Error numbers that may be returned by the kernel.
//They are defined positively here, but are returned as negative values by the kernel.
//These attempt to be the same as Linux error numbers, but please don't rely on that.

#define _SC_EPERM         1 //Operation not permitted.
#define _SC_ENOENT        2 //No such file or directory.
#define _SC_ESRCH         3 //No such process.
#define _SC_EINTR         4 //Interrupted function.
#define _SC_EIO           5 //I/O error.
#define _SC_ENXIO         6 //No such device or address.
#define _SC_E2BIG         7 //Argument list too long.
#define _SC_ENOEXEC       8 //Executable file format error.
#define _SC_EBADF         9 //Bad file descriptor.
#define _SC_ECHILD       10 //No child processes.
#define _SC_EAGAIN       11 //Resource unavailable, try again (may be the same value as EWOULDBLOCK).
#define _SC_EWOULDBLOCK  11 //Operation would block (may be the same value as EAGAIN).
#define _SC_ENOMEM       12 //Not enough space.
#define _SC_EACCES       13 //Permission denied.
#define _SC_EFAULT       14 //Bad address.
#define _SC_EBUSY        16 //Device or resource busy.
#define _SC_EEXIST       17 //File exists.
#define _SC_EXDEV        18 //Cross-device link.
#define _SC_ENODEV       19 //No such device.
#define _SC_ENOTDIR      20 //Not a directory or a symbolic link to a directory.
#define _SC_EISDIR       21 //Is a directory.
#define _SC_EINVAL       22 //Invalid argument.
#define _SC_ENFILE       23 //Too many files open in system.
#define _SC_EMFILE       24 //File descriptor value too large.
#define _SC_ENOTTY       25 //Inappropriate I/O control operation.
#define _SC_ETXTBSY      26 //Text file busy.
#define _SC_EFBIG        27 //File too large.
#define _SC_ENOSPC       28 //No space left on device.
#define _SC_ESPIPE       29 //Invalid seek.
#define _SC_EROFS        30 //Read-only file system.
#define _SC_EMLINK       31 //Too many links.
#define _SC_EPIPE        32 //Broken pipe.
#define _SC_EDOM         33 //Mathematics argument out of domain of function.
#define _SC_ERANGE       34 //Result too large.
#define _SC_EDEADLK      35 //Resource deadlock would occur.
#define _SC_EDEADLOCK    35 //Resource deadlock would occur.
#define _SC_ENAMETOOLONG 36 //Filename too long.
#define _SC_ENOLCK       37 //No locks available.
#define _SC_ENOSYS       38 //Functionality not supported.
#define _SC_ENOTEMPTY    39 //Directory not empty.
#define _SC_ELOOP        40 //Too many levels of symbolic links.
#define _SC_ENOMSG       42 //No message of the desired type.
#define _SC_EIDRM        43 //Identifier removed.

#endif //_SC_ERRNO_H
