//sc_info.h
//System status system calls
//Bryan E. Topp <betopp@betopp.com> 2022

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#ifndef _SC_INFO_H
#define _SC_INFO_H

//Enumerates processes or threads on the system.
//If "inpid" is 0, iterates through processes on the system.
//If "inpid" is positive, iterates through threads in the given process.
//Pass 0 as "prev" to begin iteration; the ID of the first process/thread is returned.
//Pass the previously returned value to continue iterating. 0 is returned when none remain.
int _sc_info_enum(int inpid, int prev);

//CPU statistics returned by the kernel
typedef struct _sc_info_cpu_s
{
	//Utilization of CPU cores, one byte each, 0 to 100.
	//Non-present CPUs are represented by 0xFF.
	unsigned char util[256];
	
} _sc_info_cpu_t;

//Outputs the latest CPU usage statistics from the kernel.
//Returns the number of bytes written or a negative error number.
int _sc_info_cpu(_sc_info_cpu_t *buf_ptr, int buf_len);

//Memory statistics returned by the kernel.
typedef struct _sc_info_mem_s
{
	//Bytes of physical memory in the system
	long long int pmem_total;
	
	//Bytes of physical memory not currently used for anything
	long long int pmem_unused;
	
	//Bytes of physical memory used for kernel features
	long long int pmem_kernel;
	
	//Amount of memory allocated for in-memory filesystem 
	long long int fs_total;
	
	//Amont of memory free in in-memory filesystem
	long long int fs_unused;	
	
} _sc_info_mem_t;

//Outputs memory statistics.
//Returns the number of bytes written or a negative error number.
int _sc_info_mem(_sc_info_mem_t *buf_ptr, int buf_len);

#endif //_SC_INFO_H
