//sc_mem.h
//System call library - memory space functions
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#ifndef _SC_MEM_H
#define _SC_MEM_H

//Adds new, private, zeroed memory to the calling process's memory space.
//Returns its address on success or a negative error number on failure.
long int _sc_mem_anon(int size);

//Adds a new displayable memory region to the calling process's memory space.
//Returns the address of the new memory on success or a negative error number on failure.
//Displayable memory regions are of sufficient size to hold one full-screen image for display.
//The correct format for the image is found by calling _sc_con_fbinfo.
long int _sc_mem_disp(void);

//Note that a process cannot remove memory from its address space, nor revoke permissions.
//This is because we don't want to support TLB shootdowns in the kernel.
//So memory that is once accessible, stays accessible, until an exec, which can only happen from a single-threaded process.

#endif //_SC_MEM_H
