//sc_sig.h
//System call library - signal handling
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#ifndef _SC_SIG_H
#define _SC_SIG_H

//Ways to alter the signal mask of a thread.
#define _SC_SIG_BLOCK 0
#define _SC_SIG_UNBLOCK 1
#define _SC_SIG_SETMASK 2

//Alters the signal mask of the calling thread.
//Consumes the signal mask from the buffer passed in. Writes the old mask in its place.
//Returns the number of bytes consumed/written or a negative error number on failure.
int _sc_sig_mask(int how, void *mask_ptr, int mask_len);

//Restores and returns to the user-mode context that was saved when the last signal was taken.
//Signals are not reentrant - if a signal handler unmasks signals, and another is taken, this call may be dysfunctional.
void _sc_sig_return(void) __attribute__((noreturn));

//Sends a signal to the given process, thread, or process group.
//Returns a nonnegative value on success or a negative error number.
int _sc_sig_send(int idtype, int id, int sig);

//Signal numbers defined in the kernel
#define _SC_SIGZERO 0
#define _SC_SIGHUP 1
#define _SC_SIGINT 2
#define _SC_SIGQUIT 3
#define _SC_SIGILL 4
#define _SC_SIGTRAP 5
#define _SC_SIGABRT 6
#define _SC_SIGEMT 7
#define _SC_SIGFPE 8
#define _SC_SIGKILL 9
#define _SC_SIGBUS 10
#define _SC_SIGSEGV 11
#define _SC_SIGSYS 12
#define _SC_SIGPIPE 13
#define _SC_SIGALRM 14
#define _SC_SIGTERM 15
#define _SC_SIGURG 16
#define _SC_SIGSTOP 17
#define _SC_SIGTSTP 18
#define _SC_SIGCONT 19
#define _SC_SIGCHLD 20
#define _SC_SIGTTIN 21
#define _SC_SIGTTOU 22
#define _SC_SIGXCPU 24
#define _SC_SIGXFSZ 25
#define _SC_SIGVTALRM 26
#define _SC_SIGPROF 27
#define _SC_SIGWINCH 28
#define _SC_SIGINFO 29
#define _SC_SIGUSR1 30
#define _SC_SIGUSR2 31
#define _SC_SIGTHR 32

#endif //_SC_SIG_H
