//sc_trace.h
//System call library - trace interface
//Bryan E. Topp <betopp@betopp.com> 2022

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#ifndef _SC_TRACE_H
#define _SC_TRACE_H

//Address spaces that can be read/written in another thread using trace calls.
typedef enum _sc_trace_space_s
{
	_SC_TRACE_SPACE_NONE     = 0, //None/invalid
	_SC_TRACE_SPACE_MEMORY   = 1, //Memory of the process containing the thread
	_SC_TRACE_SPACE_REGISTER = 2, //Saved CPU registers of the thread
	_SC_TRACE_SPACE_SIGMASK  = 3, //Signal mask of the thread
	_SC_TRACE_SPACE_SIGPEND  = 4, //Pending signals of the thread
	_SC_TRACE_SPACE_SIGREGS  = 5, //Saved registers from before a signal was taken
	_SC_TRACE_SPACE_MAX
} _sc_trace_space_t;

//Reasons that a thread can be stopped under tracing.
typedef enum _sc_trace_reason_s
{
	_SC_TRACE_REASON_NONE    = 0, //Not stopped
	_SC_TRACE_REASON_DEBUG   = 1, //Stopped at the request of a debugger
	_SC_TRACE_REASON_SIGNAL  = 2, //Stopped because a signal was taken
	_SC_TRACE_REASON_SCENTRY = 3, //Stopped while trying to make a system-call
	_SC_TRACE_REASON_SCEXIT  = 4, //Stopped while returning from a system-call
	_SC_TRACE_REASON_FORKED  = 5, //Thread is newly forked
	_SC_TRACE_REASON_EXECED  = 6, //Thread is newly exec'd
	_SC_TRACE_REASON_MAX
} _sc_trace_reason_t;

//Makes the calling thread the tracer of the given thread.
int _sc_trace_attach(int tid);

//Stops the calling thread from tracing the given thread.
int _sc_trace_detach(int tid);

//Queries or modifies the "stop reason" of the given thread.
//If reason is -1, the stop reason will be returned.
//If reason is 0, the thread will be un-stopped. The old reason is returned.
//If reason is positive, the thread will be stopped with that reason noted. The old reason is returned.
int _sc_trace_stop(int tid, _sc_trace_reason_t reason);

//Writes into the given thread. The thread must be stopped.
int _sc_trace_write(int tid, _sc_trace_space_t space, int addr, const void *src, int len);

//Reads from the given thread. The thread must be stopped.
int _sc_trace_read(int tid, _sc_trace_space_t space, int addr, void *dst, int len);

#endif //_SC_TRACE_H
