//systable.h
//System call table as macro-trick
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

//Define these symbols and then include this file for per-system-call behavior.
//SYSCALLxR, SYSCALLxV, SYSCALLxN are for system calls with x parameters.
//R means returns something, V means returns void. N means doesn't return.

SYSCALL0V(0x00, void, _sc_none          )
SYSCALL0V(0x01, void, _sc_pause         )
SYSCALL2N(0x02, void, _sc_exit,         int, int)
SYSCALL0R(0x03, int,  _sc_getpid        )
SYSCALL0R(0x04, int,  _sc_gettid        )
SYSCALL0R(0x05, int,  _sc_getppid       )
SYSCALL0R(0x06, int,  _sc_fork          )
SYSCALL0R(0x07, int,  _sc_spoon         )
SYSCALL5R(0x08, int,  _sc_wait,         int, int, int, _sc_wait_t *, int)
SYSCALL0R(0x09, int,  _sc_getticks      )

SYSCALL3R(0x20, int,  _sc_find,         int, const char *, int)
SYSCALL5R(0x21, int,  _sc_make,         int, const char *, int, int, int)
SYSCALL3R(0x22, int,  _sc_read,         int, void *, int)
SYSCALL3R(0x23, int,  _sc_write,        int, const void *, int)
SYSCALL3R(0x24, long int,  _sc_seek,    int, long int, int)
SYSCALL2R(0x25, long int,  _sc_trunc,   int, long int)
SYSCALL3R(0x26, int,  _sc_unlink,       int, const char *, int)
SYSCALL3R(0x27, int,  _sc_flag,         int, int, int)
SYSCALL1R(0x28, int,  _sc_close,        int)
SYSCALL1R(0x29, int,  _sc_chdir,        int)
SYSCALL3R(0x2a, int,  _sc_dup,          int, int, int)
SYSCALL3R(0x2b, int,  _sc_stat,         int, _sc_stat_t *, int)
SYSCALL3R(0x2c, int,  _sc_exec,         int, char * const *, char * const *)

SYSCALL2R(0x40, int,  _sc_con_fbinfo,   _sc_con_fbinfo_t *, int)
SYSCALL1R(0x41, int,  _sc_con_fbdisp,   const void *)
SYSCALL4R(0x42, int,  _sc_con_fbscale,  void *, const void *, const _sc_con_fbinfo_t *, int)
SYSCALL3R(0x43, int,  _sc_con_input,    _sc_con_input_t *, int, int)
SYSCALL1R(0x44, int,  _sc_con_pass,     int)
SYSCALL2R(0x45, int,  _sc_con_flag,     int, int)

SYSCALL1R(0x60, long int,  _sc_mem_anon,     int)
SYSCALL0R(0x61, long int,  _sc_mem_disp )

SYSCALL3R(0x80, int,  _sc_sig_mask,     int, void *, int)
SYSCALL0N(0x81, void, _sc_sig_return    )
SYSCALL3R(0x82, int,  _sc_sig_send,     int, int, int)

SYSCALL1R(0xd0, int,  _sc_trace_attach, int)
SYSCALL1R(0xd1, int,  _sc_trace_detach, int)
SYSCALL2R(0xd2, int,  _sc_trace_stop,   int, _sc_trace_reason_t)
SYSCALL5R(0xd3, int,  _sc_trace_write,  int, _sc_trace_space_t, int, const void *, int)
SYSCALL5R(0xd4, int,  _sc_trace_read,   int, _sc_trace_space_t, int, void *, int)

SYSCALL2R(0xe0, int,  _sc_info_enum,    int, int)
SYSCALL2R(0xe1, int,  _sc_info_cpu,     _sc_info_cpu_t *, int)
SYSCALL2R(0xe2, int,  _sc_info_mem,     _sc_info_mem_t *, int)

SYSCALL1N(0xee, void, _sc_panic,        const char *)

SYSCALL1R(0xff, int,  _sc_version,      int)


