//mmlibc/include/netinet/in.h
//Internet definitions for libc
//Bryan E. Topp <betopp@betopp.com> 2023
#ifndef _NETINET_IN_H
#define _NETINET_IN_H

#include <inttypes.h>
#include <sys/socket.h>
#include <arpa/inet.h>

typedef uint16_t in_port_t;
typedef uint32_t in_addr_t;

struct in_addr
{
	in_addr_t s_addr;
};

struct sockaddr_in
{
	sa_family_t sin_family;
	in_port_t sin_port;
	struct in_addr sin_addr;
};

struct in6_addr
{
	uint8_t s6_addr[16];
};

struct sockaddr_in6
{
	sa_family_t sin6_family;
	in_port_t sin6_port;
	uint32_t sin6_flowinfo;
	struct in6_addr sin6_addr;
	uint32_t sin6_scope_id;
};

extern const struct in6_addr in6addr_any;
#define IN6ADDR_ANY_INIT (struct in6_addr){0};

extern const struct in6_addr in6addr_loopback;
#define IN6ADDR_LOOPBACK_INIT (struct in6_addr){0};

struct ipv6_mreq
{
	struct in6_addr ipv6mr_multiaddr;
	unsigned ipv6mr_interface;
};

//Like BSD
#define IPPROTO_IP 0
#define IPPROTO_IPV6 41
#define IPPROTO_ICMP 1
#define IPPROTO_RAW 255
#define IPPROTO_TCP 6
#define IPPROTO_UDP 17

#define IPV6_JOIN_GROUP 12
#define IPV6_LEAVE_GROUP 13
#define IPV6_MULTICAST_HOPS 10
#define IPV6_MULTICAST_IF 9
#define IPV6_MULTICAST_LOOP 11
#define IPV6_UNICAST_HOPS 4
#define IPV6_V6ONLY 27

//Stubs
#define IN6_IS_ADDR_UNSPECIFIED(x) 0
#define IN6_IS_ADDR_LOOPBACK(x) 0
#define IN6_IS_ADDR_MULTICAST(x) 0
#define IN6_IS_ADDR_LINKLOCAL(x) 0
#define IN6_IS_ADDR_SITELOCAL(x) 0
#define IN6_IS_ADDR_V4MAPPED(x) 0
#define IN6_IS_ADDR_V4COMPAT(x) 0
#define IN6_IS_ADDR_MC_NODELOCAL(x) 0
#define IN6_IS_ADDR_MC_LINKLOCAL(x) 0
#define IN6_IS_ADDR_MC_SITELOCAL(x) 0
#define IN6_IS_ADDR_MC_ORGLOCAL(x) 0
#define IN6_IS_ADDR_MC_GLOBAL(x) 0

#define INADDR_ANY 0
#define INADDR_BROADCAST 0xFFFFFFFFu
#define INET_ADDRSTRLEN 16
#define INET6_ADDRSTRLEN 46



#endif //_NETINET_IN_H
