//mmlibc/include/sys/socket.h
//Berkeley Socket definitions for libc
//Bryan E. Topp <betopp@betopp.com> 2023
#ifndef _SYS_SOCKET_H
#define _SYS_SOCKET_H

#include <sys/uio.h>
#include <mmbits/typedef_sa_family.h>

typedef int socklen_t;

struct sockaddr
{
	sa_family_t sa_family;
	char sa_data[];
};

struct sockaddr_storage
{
	sa_family_t ss_family;
	uint64_t ss_data[16];
};

struct msghdr
{
	void *msg_name;
	socklen_t msg_namelen;
	struct iovec msg_iov;
	int msg_iovlen;
	void *msg_control;
	socklen_t msg_controllen;
	int msg_flags;
};

struct cmsghdr
{
	socklen_t cmsg_len;
	int cmsg_level;
	int cmsg_type;
	char _data[];
};

//Like BSD
#define SCM_RIGHTS 1

#define CMSG_DATA(cmsg) NULL
#define CMSG_NXTHDR(mhdr,cmsg) NULL
#define CMSG_FIRSTHDR(mhdr) NULL

struct linger
{
	int l_onoff;
	int l_linger;
};

//Like BSD
#define SOCK_DGRAM 2
#define SOCK_RAW 3
#define SOCK_SEQPACKET 5
#define SOCK_STREAM 1
#define SOL_SOCKET 65535
#define SO_ACCEPTCONN 2
#define SO_BROADCAST 32
#define SO_DEBUG 1
#define SO_DONTROUTE 16
#define SO_ERROR 4103
#define SO_KEEPALIVE 8
#define SO_LINGER 128
#define SO_OOBINLINE 256
#define SO_RCVBUF 4098
#define SO_RCVLOWAT 4100
#define SO_RCVTIMEO 4102
#define SO_REUSEADDR 4
#define SO_SNDBUF 4097
#define SO_SNDLOWAT 4099
#define SO_SNDTIMEO 4101
#define SO_TYPE 4104
#define SOMAXCONN 128
#define MSG_CTRUNC 32
#define MSG_DONTROUTE 4
#define MSG_EOR 8
#define MSG_OOB 1
#define MSG_NOSIGNAL 131072
#define MSG_PEEK 2
#define MSG_TRUNC 16
#define MSG_WAITALL 64
#define AF_INET 2
#define AF_INET6 28
#define AF_UNIX 1
#define AF_UNSPEC 0
#define SHUT_RD 0
#define SHUT_RDWR 2
#define SHUT_WR 1

int accept(int, struct sockaddr *restrict, socklen_t *restrict);
int bind(int, const struct sockaddr *, socklen_t);
int connect(int, const struct sockaddr *, socklen_t);
int getpeername(int, struct sockaddr *restrict, socklen_t *restrict);
int getsockname(int, struct sockaddr *restrict, socklen_t *restrict);
int getsockopt(int, int, int, void *restrict, socklen_t *restrict);
int listen(int, int);
ssize_t recv(int, void *, size_t, int);
ssize_t recvfrom(int, void *restrict, size_t, int, struct sockaddr *restrict, socklen_t *restrict);
ssize_t recvmsg(int, struct msghdr *, int);
ssize_t send(int, const void *, size_t, int);
ssize_t sendmsg(int, const struct msghdr *, int);
ssize_t sendto(int, const void *, size_t, int, const struct sockaddr *, socklen_t);
int setsockopt(int, int, int, const void *, socklen_t);
int shutdown(int, int);
int sockatmark(int);
int socket(int, int, int);
int socketpair(int, int, int, int [2]);

#endif //_SYS_SOCKET_H
