//basename.c
//basename implementation for libc
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <libgen.h>
#include <stddef.h>
#include <string.h>

char *basename(char *path)
{
	//Handle empty case
	if((path == NULL) || (strlen(path) < 1))
	{
		static char dot[2] = {'.', '\0'};
		return dot;
	}
	
	//Delete trailing slashes
	char *end = path + strlen(path);
	while(end > path)
	{
		if(*end == '/' || *end == '\0')
			*end = '\0';
		else
			break;
		
		end--;
	}
	
	if(path[0] == '/' && path[1] == '\0')
		return path; //Path was entirely slashes
	
	//Advance through slashes until none remain
	while(1)
	{
		char *slash = strchr(path, '/');
		if(slash == NULL)
			return path;
		
		path = slash;
		while(*path == '/')
			path++;
	}
}

char *dirname(char *path)
{
	//Handle empty case
	if((path == NULL) || (strlen(path) < 1) || !strchr(path, '/'))
	{
		static char dot[2] = {'.', '\0'};
		return dot;
	}
	
	//Delete trailing slashes
	char *end = path + strlen(path);
	while(end > path)
	{
		if(*end == '/' || *end == '\0')
			*end = '\0';
		else
			break;
		
		end--;
	}
	
	if(path[0] == '/' && path[1] == '\0')
		return path; //Path was entirely slashes
	
	//Turn the last non-trailing slash into a NUL
	while(end > path)
	{
		end--;
		if(*end == '/')
		{
			*end = '\0';
			return path;
		}
	}
	
	return path;
}
