//clocks.c
//Time counting in libc
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <time.h>
#include <sys/time.h>
#include <errno.h>
#include <sc.h>
#include <string.h>


int clock_gettime(clockid_t clock_id, struct timespec *tp)
{
	(void)clock_id;
	(void)tp;
	
	errno = ENOSYS;
	return -1;
}

unsigned int sleep(unsigned int seconds)
{
	int64_t ticks_needed = _sc_getticks() + (seconds * 1000l);
	while(_sc_getticks() < ticks_needed)
	{
		_sc_pause();
	}
	return 0;
}

unsigned int alarm(unsigned int seconds)
{
	(void)seconds;
	return 0;
}

char *ctime_r(const time_t *clock, char *buf)
{
	(void)clock;
	strcpy(buf, "Unk Unk 00 00:00:00 0000\n");
	return buf;
}

char _ctime_buffer[26];
char *ctime(const time_t *clock)
{
	return ctime_r(clock, _ctime_buffer);
}


int sched_yield(void)
{
	_sc_pause();
	return 0;
}

int nanosleep(const struct timespec *rqtp, struct timespec *rmtp)
{
	uint64_t start = _sc_getticks();
	
	uint64_t ms = rqtp->tv_sec * 1000ul;
	ms += rqtp->tv_nsec / 1000000ul;
	
	uint64_t end;
	while(1)
	{
		end = _sc_getticks();
		if(start + ms <= end)
			break;
		
		_sc_pause();
	}
	
	rmtp->tv_sec = (end - start) / 1000ul;
	rmtp->tv_nsec = ((end - start) % 1000ul) * 1000000ul;
	return 0;
}

int utimensat(int fd, const char *path, const struct timespec times[2], int flag)
{
	(void)fd;
	(void)path;
	(void)times;
	(void)flag;
	errno = ENOSYS;
	return -1;
}

char *strptime(const char * restrict buf, const char * restrict format, struct tm * restrict timeptr)
{
	(void)buf;
	(void)format;
	(void)timeptr;
	return NULL;
}

void tzset(void)
{
	return;
}

int futimens(int fd, const struct timespec times[2])
{
	(void)fd;
	(void)times;
	errno = ENOSYS;
	return -1;
}

struct tm _gmtime_buf = {0};
struct tm *gmtime(const time_t *clock)
{
	return gmtime_r(clock, &_gmtime_buf);
}

struct tm *gmtime_r(const time_t *clock, struct tm *result)
{
	memset(result, 0, sizeof(*result));
	(void)clock;
	errno = ENOSYS;
	return result;
}

int settimeofday(const struct timeval *tp, const void *tzp)
{
	(void)tp;
	(void)tzp;
	errno = ENOSYS;
	return -1;
}




