//entry.c
//libc entry point
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <sc.h>
#include <stdint.h>

char **_default_environ = (char*[]){ "STUMP=" BUILDVERSION , NULL };

void _libc_entry(int argc, char **argv, char **envp)
{
	//Count argv if we need to
	if(argc == 0)
	{
		while(argv != NULL && argv[argc] != NULL)
			argc++;
	}
	
	//Set aside initial environment location
	extern char **environ;
	if(envp != NULL)
		environ = envp;
	else
		environ = _default_environ;
	
	//Tell kernel about what system-call interface we expect
	//Todo - handle older versions if we lack some system-calls or something
	int32_t our_version = 1;
	int32_t version_to_use = _sc_version(our_version);
	(void)version_to_use;
	
	//Set signal actions to default
	for(int ss = 0; ss < 64; ss++)
	{
		signal(ss, SIG_DFL);
	}
	int64_t none_masked = 0;
	_sc_sig_mask(SIG_SETMASK, &none_masked, sizeof(none_masked));
	
	//Call constructors
	extern void _init();
	_init();
	
	//Call main
	extern int main();
	int main_returned = main(argc, argv, envp);
	
	//If main returns, call exit with its return value.
	exit(main_returned);
	
	//Exit shouldn't return
	_Exit(main_returned);
	abort();
	while(1){}
}
