//exit.c
//Exit and at-exit functions in libc
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <stdlib.h>
#include <errno.h>
#include <sc.h>

typedef struct _atexit_s
{
	void (*function)(void);
} _atexit_t;
#define _ATEXIT_MAX 64
_atexit_t _atexit_array[_ATEXIT_MAX];
int _atexit_count;

void _atexit_run(void)
{
	//Pop all at-exit functions off the list, calling the function and freeing each entry.
	while(_atexit_count > 0)
	{
		_atexit_count--;
		_atexit_t *a = &(_atexit_array[_atexit_count]);
		a->function();
	}
}

void exit(int status)
{
	//Call registered atexit() functions
	_atexit_run();
	
	//Call destructors
	extern void _fini();
	_fini();
	
	//Todo - flush streams, close output streams, delete temp files
	_Exit(status);
}

void _Exit(int status)
{
	_sc_exit(status, 0);
	
	//That shouldn't return, but we really want to crash if it does.
	abort();
	while(1) { (*(volatile int*)0)++; }
}

void _exit(int status)
{
	_Exit(status);
}

int atexit(void (*function)(void))
{
	if(_atexit_count < _ATEXIT_MAX)
	{
		_atexit_array[_atexit_count].function = function;
		_atexit_count++;
	}
	else
	{
		errno = ENOMEM;
		return -1;
	}
	
	return 0;
}

void abort(void)
{
//	while(1) { } //catch before kernel is ready to
	raise(SIGABRT);
	_sc_exit(-1, SIGABRT);
}
