//id.c
//Process/user ID functions for libc
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sc.h>

pid_t getpid(void)
{
	return _sc_getpid();
}

pid_t getppid(void)
{
	return _sc_getppid();
}

int gethostname(char *name, size_t namelen)
{
	static const char * const hostname = "localhost";
	if(namelen < strlen(hostname))
	{
		errno = ENAMETOOLONG;
		return -1;
	}
	
	strncpy(name, hostname, namelen);
	return 0;
}

pid_t getpgrp(void)
{
	return getpgid(0);
}

pid_t getpgid(pid_t pid)
{
	(void)pid;
	return 0;
}

int setpgid(pid_t pid, pid_t pgrp)
{
	(void)pid;
	(void)pgrp;
	return -ENOSYS;
}

pid_t getsid(pid_t pid)
{
	//one session
	(void)pid;
	return 0;
}

gid_t getgid(void)
{
	//one group
	return 0;
}

gid_t getegid(void)
{
	//one group
	return 0;
}

uid_t getuid(void)
{
	//one user
	return 0;
}

uid_t geteuid(void)
{
	//one user
	return 0;
}

int setgid(gid_t gid)
{
	if(gid != 0)
	{
		errno = ENOSYS;
		return -1;
	}
	return 0;
}

int setegid(gid_t gid)
{
	if(gid != 0)
	{
		errno = ENOSYS;
		return -1;
	}
	return 0;
}

int setuid(uid_t uid)
{
	if(uid != 0)
	{
		errno = ENOSYS;
		return -1;
	}
	return 0;
}

int seteuid(uid_t uid)
{
	if(uid != 0)
	{
		errno = ENOSYS;
		return -1;
	}
	return 0;
}
