//mktemp.c
//Temporary file functions in libc
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>

//Returns a random alphanumeric character for use in a temporary filename.
static char _mktemp_rand(void)
{
	int r = rand() % 62;
	if(r < 26)
		return 'a' + r;
	else if(r < 52)
		return 'A' + (r - 26);
	else
		return '0' + (r - 52);
}

char *mktemp(char *tpl)
{
	//Handle NULL or zero-length strings trivially
	if(tpl == NULL || *tpl == '\0')
		return NULL;
	
	//Work backwards from the last character, replacing trailing X with random alphanum.
	int replaced = 0;
	char *cc = tpl + strlen(tpl) - 1;
	while(cc >= tpl)
	{
		if(*cc != 'X')
			break;
		
		*cc = _mktemp_rand();		
		cc--;
		replaced++;
	}
	
	if(replaced > 0)
		return tpl;
	else
		return NULL;
}

int mkstemp(char *tpl)
{
	char *name = mktemp(tpl); //Probably in the same buffer, ultimately
	if(name == NULL)
	{
		errno = EINVAL;
		return -1;
	}
	
	int fd = open(tpl, O_CREAT | O_EXCL, 0600);
	return fd; //open sets errno and returns -1 on failure
}

char *mkdtemp(char *tpl)
{
	char *retval = mktemp(tpl);
	if(mkdir(retval, 700) != 0)
		return NULL;
	return retval;
}
