//net.c
//Berkeley Socket stubs in libc
//Bryan E. Topp <betopp@betopp.com> 2023

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <stddef.h>
#include <sys/socket.h>

int getsockname(int s, struct sockaddr * restrict name, socklen_t * restrict namelen)
{
	(void)s;
	(void)name;
	(void)namelen;
	errno = ENOSYS;
	return -1;
}

int getpeername(int s, struct sockaddr * restrict name, socklen_t * restrict namelen)
{
	(void)s;
	(void)name;
	(void)namelen;
	errno = ENOSYS;
	return -1;
}

int bind(int s, const struct sockaddr *addr, socklen_t addrlen)
{
	(void)s;
	(void)addr;
	(void)addrlen;
	errno = ENOSYS;
	return -1;
}

int connect(int s, const struct sockaddr *name, socklen_t namelen)
{
	(void)s;
	(void)name;
	(void)namelen;
	errno = ENOSYS;
	return -1;
}

int socket(int domain, int type, int protocol)
{
	(void)domain;
	(void)type;
	(void)protocol;
	errno = ENOSYS;
	return -1;
}

ssize_t recvfrom(int s, void *buf, size_t len, int flags, struct sockaddr * restrict from, socklen_t * restrict fromlen)
{
	(void)s;
	(void)buf;
	(void)len;
	(void)flags;
	(void)from;
	(void)fromlen;
	errno = ENOSYS;
	return -1;
}

int listen(int s, int backlog)
{
	(void)s;
	(void)backlog;
	errno = ENOSYS;
	return -1;
}

int shutdown(int s, int how)
{
	(void)s;
	(void)how;
	errno = ENOSYS;
	return -1;
}

int setsockopt(int s, int level, int optname, const void *optval, socklen_t optlen)
{
	(void)s;
	(void)level;
	(void)optname;
	(void)optval;
	(void)optlen;
	errno = ENOSYS;
	return -1;
}

int getaddrinfo(const char *hostname, const char *servname, const struct addrinfo *hints, struct addrinfo **res)
{
	(void)hostname;
	(void)servname;
	(void)hints;
	(void)res;
	errno = ENOSYS;
	return EAI_SYSTEM;
}

void freeaddrinfo(struct addrinfo *ai)
{
	(void)ai;
	return;
}

int accept(int s, struct sockaddr * restrict addr, socklen_t * restrict addrlen)
{
	(void)s;
	(void)addr;
	(void)addrlen;
	errno = ENOSYS;
	return -1;
}

const char *inet_ntop(int af, const void * restrict src, char * restrict dst, socklen_t size)
{
	(void)af;
	(void)src;
	(void)dst;
	(void)size;
	errno = ENOSYS;
	return NULL;
}

const char * const _gai_strerror_strings[] = 
{
	[EAI_AGAIN] = "The name could not be resolved at this time. Future attempts may succeed.",
	[EAI_BADFLAGS] = "The flags had an invalid value.",
	[EAI_FAIL] = "A non-recoverable error occurred.",
	[EAI_FAMILY] = "The address family was not recognized or the address length was invalid for the specified family.",
	[EAI_MEMORY] = "There was a memory allocation failure.",
	[EAI_NONAME] = "The name does not resolve for the supplied parameters.",
	[EAI_SERVICE] = "The service passed was not recognized for the specified socket type.",
	[EAI_SOCKTYPE] = "The intended socket type was not recognized.",
	[EAI_SYSTEM] = "A system error occurred. The error code can be found in errno.",
	[EAI_OVERFLOW] = "An argument buffer overflowed.",
};

const char *gai_strerror(int ecode)
{
	const char *fallback = "Unknown error.";
	
	if(ecode < 0)
		return fallback;
	
	if(ecode >= (int)(sizeof(_gai_strerror_strings)/sizeof(_gai_strerror_strings[0])))
		return fallback;
	
	if(_gai_strerror_strings[ecode] == NULL)
		return fallback;
	
	return _gai_strerror_strings[ecode];
}
