//nosys.c
//Unimplemented features that fail immediately in libc
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <sys/mman.h>


void _dummy(void)
{
	
}

int chown(const char *path, uid_t owner, gid_t group)
{
	//Look up the supplied path and pass it on to fchown.
	int fd = open(path, O_RDWR | O_CLOEXEC);
	if(fd < 0)
		return -1; //open sets errno
	
	int result = fchown(fd, owner, group);
	close(fd);
	return result;
}

int fchown(int fd, uid_t owner, gid_t group)
{
	return fchownat(fd, "", owner, group, 0);
}

int lchown(const char *path, uid_t owner, gid_t group)
{
	return fchownat(AT_FDCWD, path, owner, group, AT_SYMLINK_NOFOLLOW);
}

int fchownat(int fd, const char *path, uid_t owner, gid_t group, int flag)
{
	(void)path;
	(void)flag;
	
	//This is a single-user system. All files are owned by uid/gid 0.
	//Allow trivial cases of keeping that setting - fail all others.
	if(fd < 0 && fd != AT_FDCWD)
	{
		errno = EBADF;
		return -1;
	}
	
	if(owner != 0 && owner != -1)
	{
		errno = ENOSYS;
		return -1;
	}
	
	if(group != 0 && group != -1)
	{
		errno = ENOSYS;
		return -1;
	}
	
	return 0;
}

void sync(void)
{
}

int chroot(const char *dirname)
{
	//We do intend to support this, but later
	(void)dirname;
	abort();
}

void *mmap(void *addr, size_t len, int prot, int flags, int fd, off_t offset)
{
	(void)addr;
	(void)len;
	(void)prot;
	(void)flags;
	(void)fd;
	(void)offset;
	errno = ENOSYS;
	return MAP_FAILED;
}

int munmap(void *addr, size_t len)
{
	(void)addr;
	(void)len;
	errno = ENOSYS;
	return -1;
}


int getpriority(int which, int who)
{
	(void)which;
	(void)who;
	errno = ENOSYS;
	return -1;
}

int setpriority(int which, int who, int prio)
{
	(void)which;
	(void)who;
	(void)prio;
	errno = ENOSYS;
	return -1;
}

