//passwd.c
//Password database stubs in libc
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <pwd.h>
#include <grp.h>
#include <stddef.h>
#include <string.h>
#include <errno.h>

static struct passwd _passwd_buf;
static struct group _group_buf;

int getpwuid_r(uid_t uid, struct passwd *pwd, char *buffer, size_t bufsize, struct passwd **result)
{
	return getpwnam_r((uid == 0)?"root":"invalid", pwd, buffer, bufsize, result);
}

struct passwd *getpwuid(uid_t uid)
{
	return getpwnam((uid==0)?"root":"invalid");
}

int getgrgid_r(gid_t gid, struct group *grp, char *buffer, size_t bufsize, struct group **result)
{
	return getgrnam_r((gid==0)?"root":"invalid", grp, buffer, bufsize, result);
}

struct group *getgrgid(gid_t gid)
{
	return getgrnam((gid==0)?"root":"invalid");
}

struct passwd *getpwnam(const char *login)
{
	struct passwd *result = NULL;
	if(getpwnam_r(login, &_passwd_buf, NULL, 0, &result) != 0)
		return NULL;
	
	return result;
}

struct group *getgrnam(const char *name)
{
	struct group *result = NULL;
	if(getgrnam_r(name, &_group_buf, NULL, 0, &result) != 0)
		return NULL;
	
	return result;
}

int getpwnam_r(const char *name, struct passwd *pwd, char *buffer, size_t bufsize, struct passwd **result)
{
	(void)buffer;
	(void)bufsize;
	
	//Stub for single-user for now
	if(name == NULL || strcmp(name, "root") != 0)
	{
		*result = NULL;
		return 0;
	}
	
	static char rootname[5] = { 'r', 'o', 'o', 't', '\0' };
	pwd->pw_name = rootname;
	pwd->pw_uid = 0;
	pwd->pw_gid = 0;
	static char rootdir[2] = { '/', '\0' };
	pwd->pw_dir = rootdir;
	static char rootshell[10] = { '/', 'b', 'i', 'n', '/', 'o', 'k', 's', 'h', '\0' };
	pwd->pw_shell = rootshell;
	
	*result = pwd;
	return 0;
}

int getgrnam_r(const char *name, struct group *grp, char *buffer, size_t bufsize, struct group **result)
{
	(void)buffer;
	(void)bufsize;
	
	//Stub for single-user for now
	if(name == NULL || strcmp(name, "root") != 0)
	{
		*result = NULL;
		return 0;
	}
	
	static char rootname[5] = { 'r', 'o', 'o', 't', '\0' };
	grp->gr_name = rootname;
	grp->gr_gid = 0;
	
	static char *rootmem[2] = { rootname, NULL };
	grp->gr_mem = rootmem;

	*result = grp;
	return 0;
}

int getgroups(int gidsetlen, gid_t *gidset)
{
	if(gidsetlen < 1)
	{
		errno = EINVAL;
		return -1;
	}
	
	gidset[0] = 0;
	return 1;
}
