//rtc.c
//Realtime clock functions in libc
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <time.h>
#include <errno.h>
#include <sc.h>

//Temp
char *tzname[2] = { "UTC", "UTC" };

time_t time(time_t *tloc)
{
	//This system specifically doesn't have an RTC so it can't snitch on you.
	//However, some applications might expect time() to be monotonic and increasing.
	//So just count seconds since we booted.
	//(Maybe later we'll support a libc-level "set RTC" call or something. Env var?)
	time_t sec = _sc_getticks() / 1000l;
	if(tloc != NULL)
		*tloc = sec;
	
	return sec;
}

struct tm _localtime_buf;
struct tm *localtime(const time_t *clock)
{
	return localtime_r(clock, &_localtime_buf);
}

struct tm *localtime_r(const time_t *clock, struct tm *result)
{
	//This is really wrong but I don't care right now
	int64_t remain = *clock;
	
	result->tm_sec = remain % 60;
	remain /= 60;
	result->tm_min = remain % 60;
	remain /= 60;
	result->tm_hour = remain % 24;
	remain /= 24;
	result->tm_yday = remain % 365;
	remain /= 365;
	result->tm_year = remain + 70;	
	result->tm_mday = 0;
	result->tm_mon = 0;
	result->tm_wday = 0;
	result->tm_isdst = 0;
	result->tm_zone = tzname[0];
	result->tm_gmtoff = 0;

	return result;
}

time_t mktime(struct tm *tm)
{
	//Super temp too
	return (((((((tm->tm_year * 365) + tm->tm_yday) * 24) + tm->tm_hour) * 60) + tm->tm_min) * 60) + tm->tm_sec;
}

