//setlogin.c
//Temp implementation of setlogin in libc
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <errno.h>
#include <string.h>

//For now we're single-user so all processes have permission to "change their login name".
//So just keep this in memory.
char _login[32] = "root";

char *getlogin(void)
{
	return _login;
}

int getlogin_r(char *name, size_t len)
{
	if(strlen(_login) >= len)
		return ERANGE;
	
	strncpy(name, _login, len);
	return 0;
}

int setlogin(const char *name)
{
	if(strlen(name) >= sizeof(_login))
	{
		errno = EINVAL;
		return -1;
	}
	
	strncpy(_login, name, sizeof(_login)-1);
	return 0;
}
