//mmlibc/src/statvfs.c
//Filesystem usage functions for MMK's libc.
//Bryan E. Topp <betopp@betopp.com> 2023

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <sys/statvfs.h>
#include <sc.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

int statvfs(const char * restrict path, struct statvfs * restrict buf)
{
	int fd = open(path, O_CLOEXEC | O_RDONLY);
	if(fd < 0)
		return -1; //open sets errno
	
	int result = fstatvfs(fd, buf);
	close(fd);
	return result;
}

int fstatvfs(int fd, struct statvfs *buf)
{
	//All files are in the same filesystem - in RAM.
	(void)fd;
	
	//Get memory info for info about the filesystem.
	_sc_info_mem_t m = {0};
	int result = _sc_info_mem(&m, sizeof(m));
	if(result < 0)
	{
		errno = -result;
		return -1;
	}
	
	memset(buf, 0, sizeof(*buf));
	
	//Assume 512-byte blocks
	buf->f_bsize = 512;
	buf->f_frsize = 512;
	
	//Compute number of "blocks" based on bytes total/available
	buf->f_blocks = m.fs_total / buf->f_frsize;
	buf->f_bfree = m.fs_unused / buf->f_frsize;
	buf->f_bavail = buf->f_bfree;
	
	//No limit on inodes in our system - they're allocated like data blocks
	buf->f_files = 0;
	buf->f_ffree = -1ull;
	buf->f_favail = -1ull;
	
	//Always filesystem ID 0, never support setuid
	buf->f_fsid = 0;
	buf->f_flag = ST_NOSUID;
	
	//Max filename length is determined by directory entries, as represented in system-call ABI.
	_sc_dirent_t de;
	buf->f_namemax = sizeof(de.name) - 1;	
	
	buf->f_type = 0x01021994ul;
	
	return 0;
}

