//termios.c
//Terminal control functions in libc
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <termios.h>
#include <sc.h>
#include <errno.h>
#include <string.h>

int tcgetattr(int fd, struct termios *t)
{	
	//Todo - maybe have /dev/ttyXctl in addition to /dev/ttyX?
	//I like how our kernel implements pseudoterminals as straight pipes.
	(void)fd;
	(void)t;
	errno = ENOSYS;
	return -1;
}

int tcsetattr(int fd, int action, const struct termios *t)
{
	(void)fd;
	(void)action;
	(void)t;
	errno = ENOSYS;
	return -1;
}

pid_t tcgetpgrp(int fd)
{
	(void)fd;
	errno = ENOSYS;
	return -1;
}

int tcsetpgrp(int fd, pid_t pgrp_id)
{
	(void)fd;
	(void)pgrp_id;
	errno = ENOSYS;
	return -1;
}

speed_t cfgetispeed(const struct termios *t)
{
	return t->c_ispeed;
}

int cfsetispeed(struct termios *t, speed_t speed)
{
	t->c_ispeed = speed;
	return 0;
}

speed_t cfgetospeed(const struct termios *t)
{
	return t->c_ospeed;
}

int cfsetospeed(struct termios *t, speed_t speed)
{
	t->c_ospeed = speed;
	return 0;
}

void cfmakeraw(struct termios *t)
{
	//As given in Linux manual.
	t->c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL | IXON);
	t->c_oflag &= ~(OPOST);
	t->c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);
	t->c_cflag &= ~(CSIZE | PARENB);
	t->c_cflag |= CS8;
}

int tcsendbreak(int fd, int len)
{
	(void)fd;
	(void)len;
	errno = ENOSYS;
	return -1;
}
