//ttyname.c
//ttyname for libc
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <sc.h>
#include <sys/stat.h>

char _ttyname_buf[32];
char *ttyname(int fd)
{
	int result = ttyname_r(fd, _ttyname_buf, sizeof(_ttyname_buf) - 1);
	if(result == 0)
	{
		return _ttyname_buf;
	}
	else
	{
		errno = result;
		return NULL;
	}
}

int ttyname_r(int fd, char *buf, size_t len)
{
	(void)fd;
	strncpy(buf, "unknown", len);
	return ENOSYS;
}

int isatty(int fd)
{
	struct stat st = {0};
	int st_result = fstat(fd, &st);
	
	//Return false if we can't stat the given file
	if(st_result == -1)
		return 0;
		
	//Actual terminal devices in this system are a type of file (like symlinks or named-pipes)
	//We call them pseudoterminal files.
	if(S_ISPTY(st.st_mode))
		return 1;
	
	//Also consider character-devices to be terminals
	if(S_ISCHR(st.st_mode))
		return 1;
	
	return 0;
}
