//mmlibc/src/utsname.c
//System name implementation for libc
//Bryan E. Topp <betopp@betopp.com> 2023

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <sys/utsname.h>
#include <stdlib.h>
#include <string.h>

int uname(struct utsname *u)
{
	//Allow environment overrides like BSD.
	//In Stump we'll typically set these from PID1.
	char *sysname = getenv("UNAME_s");
	if(sysname == NULL)
		sysname = "stump";
	
	char *nodename = getenv("UNAME_n");
	if(nodename == NULL)
		nodename = "localhost";
	
	char *release = getenv("UNAME_r");
	if(release == NULL)
		release = "debug";
	
	char *version = getenv("UNAME_v");
	if(version == NULL)
		version = "0.0.0";
	
	char *machine = getenv("UNAME_m");
	if(machine == NULL)
	{
		#if defined(__amd64__) || defined(__amd64__) || defined(_M_AMD64)
		machine = "amd64";
		#elif defined(__arm__) || defined(_ARM) || defined(_M_ARM)
		machine = "arm";
		#else
		machine = "unknown";
		#endif
	}
	
	strncpy(u->sysname, sysname, sizeof(u->sysname)-1);
	strncpy(u->nodename, nodename, sizeof(u->nodename)-1);
	strncpy(u->release, release, sizeof(u->release)-1);
	strncpy(u->version, version, sizeof(u->version)-1);
	strncpy(u->machine, machine, sizeof(u->machine)-1);
	return 0;
}


