//wait.c
//Child status functions in libc
//Bryan E. Topp <betopp@betopp.com> 2021

//This file is part of Stump; Copyright Bryan E. Topp, 2023.
//Stump is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//Stump is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with Stump. If not, see <https://www.gnu.org/licenses/>. 

#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <unistd.h>
#include <sc.h>

pid_t wait(int *status)
{
	return waitpid(-1, status, 0);
}

pid_t waitpid(pid_t wpid, int *status, int options)
{
	idtype_t wait_idtype = P_ALL;
	int64_t wait_id = 0;
	if(wpid == -1)
	{
		wait_idtype = P_ALL;
		wait_id = 0;
	}
	else if(wpid == 0)
	{
		wait_idtype = P_PGID;
		wait_id = getpgrp();
	}
	else if(wpid > 0)
	{
		wait_idtype = P_PID;
		wait_id = wpid;
	}
	else if(wpid < -1)
	{
		wait_idtype = P_PGID;
		wait_id = -wpid;
	}
	
	options |= WEXITED;
	
	//_sc_wait in our kernel doesn't actually support waiting.
	//It always behaves like WNOHANG in a normal kernel.
	//So if we want to really wait here, check for status in a loop and _sc_pause inbetween.
	while(1)
	{
		_sc_wait_t w = {0};
		ssize_t result = _sc_wait(wait_idtype, wait_id, options, &w, sizeof(w));
		if(result > 0)
		{
			*status = w.status;
			return w.pid;
		}
		else if(result < 0)
		{
			errno = -result;
			return -1;
		}
		else if(options & WNOHANG)
		{
			return 0;
		}
		
		_sc_pause();
	}
}

int nice(int incr)
{	
	(void)incr;
	errno = ENOSYS;
	return -1;
}
